<?php

/* channel.lib.php
   Defines a class used for managing Siteation channels.
   Part of Siteation.
   Copyright (C) 2005 Studio 703.  All rights reserved.
*/

require_once("site.error.php");
require_once("query.lib.php");
require_once("site.lib.php");

// This class handles Siteation channels
class SITE_CHANNEL {
  // Stores the site handler used to manage the site
  private $site;
  // Stores the data source used to manage the channel information
  private $dataSource;
  /* Indicates whether or not the channel information has changed since it was
     last read from or written to the database */
  private $changed;
  // Stores fields retrieved from the channel's record
  private $settings;
  
  // Constructor
  function __construct($siteHandler) {
    // Continue only if we have a valid-looking site handler
    if (!isset($siteHandler)) return(seInvalidParam);
    // Store the site handler for future use
    $this->site = $siteHandler;
    // Store the site data source handler for future use
    $this->dataSource = $this->site->DataSource();
  }
  
  // Frees the channel handler and flushes any changes to the database
  function __destruct() {
    // Flush any changes to the database
    $this->FlushChanges();
  }
  
  /* Retrieves the specified channel setting. */
  function Setting($settingName) {
    // Continue only if we have a valid setting
    if (empty($settingName) || !isset($this->settings[$settingName]))
      return(NULL);
    // Return the requested setting
    return($this->settings[$settingName]);
  }

  // Sets the specified channel setting
  function Set($settingName, $value) {
    // Continue only if we have a valid-looking setting
    if (empty($settingName) || !isset($this->settings[$settingName]))
      return(seInvalidParam);
    // Store the new setting
    $this->settings[$settingName] = $value;
    // Mark the channel information as changed
    $this->changed = TRUE;
    // Return success
    return(seOkay);
  }

  // Retrieves all of the channel settings at once
  function Settings() {
    // Return all channel settings
    return($this->settings);
  }

  // Sets all of the channel settings at once
  function SetAll($settings) {
    // Store the new settings
    $this->settings = $settings;
    // Mark the channel information as changed
    $this->changed = TRUE;
  }

  // Retrieves the specified channel from the database
  function Get($channelName) {
    // Continue only if we have a valid-looking channel name
    if (empty($channelName)) return(seInvalidParam);
    global $SITE_CONFIG;
    // Create a query
    $channel = new QUERY($this->dataSource);
    // Select the table to search
    $channel->location = array(
      $SITE_CONFIG["TABLE_PREFIX"]."channels" => "*");
    // Set the criteria
    $channel->criteria = array(
      "ChannelName=", Quote($channelName));
    // Run the query
    $result = $channel->Select();
    // If the query failed, indicate that failure
    if ($result <= 0) return($result);
    // Retrieve the row containing the channel information
    $this->settings = $channel->NextRowVerbose();
    // Mark the channel information as unchanged
    $this->changed = FALSE;
    // Free the query
    $channel->__destruct();
    // Indicate success
    return($this->settings["ID"]);
  }

  // Flushes session information to the database
  function FlushChanges() {
    // Continue only if changes were made
    if (!$this->changed) return(seOkay);
    global $SITE_CONFIG;
    // Create a query to update the channel information
    $channel = new Query($this->dataSource);
    // Set the table on which to operate
    $channel->location = array($SITE_CONFIG["TABLE_PREFIX"]."channels");
    // Set the fields and values to be updated
    $channel->fieldsValues = $this->settings;
    // Set the criteria so that the correct channel record is updated
    $channel->criteria = array(
      "ID =", $this->settings["ID"]);
    // Run the query
    $result = $channel->Update();
    // If the query failed, return the error
    if ($result < 0) return($result);
    // Mark the channel data as unchanged
    $this->changed = FALSE;
    // Return success
    return(seOkay);
  }

  // Retrieves a list of views that apply to this channel
  function GetViews() {
    // Continue only if we know the channel name
    if (!isset($this->settings["ChannelName"]) ||
      empty($this->settings["ChannelName"])) return(NULL);
    global $SITE_CONFIG;
    // Create a query
    $view = new QUERY($this->dataSource);
    // Set the table to be searched and the fields to return
    $view->location = array(
      $SITE_CONFIG["TABLE_PREFIX"]."views" => array("ID", "ViewName"));
    // Set the criteria
    $view->criteria = array(
      "ChannelName" => "=".Quote($this->settings["ChannelName"]));
    // Set the sort order
    $view->sort = array("ViewName");
    // Run the query
    $result = $view->Select();
    // If the query failed, return the error
    if ($result < 0) return($result);
    // This array will hold our results
    $viewList = NULL;
    // Retrieve the first row of results
    $resultRow = $view->NextRowVerbose();
    // Get every row of results
    while($resultRow) {
      // Store the view ID and view name
      $viewList[$resultRow["ViewName"]] = $resultRow["ID"];
      // Get the next row
      $resultRow = $view->NextRowVerbose();
    }
    // Free the query
    $view->__destruct();
    // Return the resulting list
    return($viewList);
  }

}

?>
