<?php

/* mssql.lib.php
   Provides an interface to a Microsoft SQL server instance.
   Part of Siteation.
   Copyright (C) 2005 Studio 703.  All rights reserved.
*/

require_once("site.error.php");
require_once("datasource.lib.php");
require_once("misc.lib.php");

// This class interfaces with a Microsoft SQL server instance
class MSSQLDataSource extends DATA_SOURCE {
  // Constructor
  function __construct($host, $database) {
    // Store the name of the database server
    $this->host = $host;
    // Store the name of the database to be accessed
    $this->dataSource = $database;
  }
  
  // Connects to the SQL server using the specified user name and password
  function Connect($userName, $password) {
    // Continue only if we were passed a valid-looking user name
    if (empty($userName)) return(seInvalidParam);
    try {
      // Store the username for future use
      $this->userName = $userName;
      $dsn = "Provider='sqloledb';";
      $dsn .= "Data Source='".$this->host."';";
      $dsn .= "Initial Catalog='".$this->dataSource."';";
      $dsn .= "uid=".$this->userName.";";
      $dsn .= "pwd=".$password.";";
      $this->handle = new COM("ADODB.Connection");
      $this->handle->Open($dsn);
    }
    catch(Exception $e) {
      // If we failed, return an error
      DiePrettily(seDataSource, array($this->host, $e->getMessage()));
    }
    DebugOut("DATASOURCE", "Successfully connected to ".$this->host);
    return(seOkay);
  }
  
  // Executes the specified query string
  function Query($queryString) {
    // Ensure we have a valid-looking query string
    if (empty($queryString)) return(null);
    try {
      // Stash the query string for future reference
      $this->queryString = $queryString;
      $recordset = new COM("ADODB.Recordset");
      // Start timing the query
      $queryStart = microt(TRUE);
      // Send the query string to the server
      //DebugOut("QUERY", "DSN: ".$dsn);
      $recordset->Open($this->queryString, $this->handle, 1, 2, 1);
      // Stop timing the query
      $this->queryRunTime = round(microt(TRUE) - $queryStart, 4);
      // Output a debug message (if we're debugging)
      DebugOut("QUERY", "[".$this->queryRunTime." second(s)] - ".Quote($queryString,"")." returned ".($recordset ? $recordset:"NOTHING\n".@mssql_get_last_message()));
      // Update the query counter
      $this->queryCount++;
    }
    catch (Exception $e) {
      DiePrettily(seQuery, array($this->queryString, $e->getMessage()));
    }
    // Return success
    return $recordset;
  }
  
  // Executes a SELECT query on the current database
  function Select($query) {
    // Continue only if we have a valid looking list of tables to search
    if (empty($query->location)) return(seInvalidParam);
    // Start constructing the query string
    $queryString = "SELECT ";
    // If a limit was specified, include it
    if (!empty($query->limit)) $queryString .= "TOP ".$query->limit." ";
    // Add the list of fields to be retrieved from each table
    foreach($query->location as $tableName => $fieldList) {
      // If more than one field is specified, retrieve all of the fields
      if (is_array($fieldList)) {
        foreach($fieldList as $fieldName) {
          // Remove extra spaces from the field name
          $fieldName = trim($fieldName);
          if ($fieldName != "*") $fieldName = "[".$fieldName."]";
          $queryString.= $fieldName.",";
        }
        // Remove the trailing comma from the query string
        $queryString = rtrim($queryString, ",");
      }
      // Otherwise, add the field list directly to the string
      else $queryString .= $fieldList;
    }
    // Add the list of tables to the query string
    $queryString .= " FROM ".implode(",", array_keys($query->location));
    // If filter criteria was specified for this search, include it
    if (isset($query->criteria) && !empty($query->criteria)) {
      $queryString .= " WHERE ";
      // If the criteria are specified in an array, get them as a string
      if (is_array($query->criteria))
        $queryString .= implode_assoc($query->criteria, " ", " ");
      // Otherwise, add the criteria directly to the query string
      else $queryString .= $query->criteria;
    }
    // If sort criteria was specified, include it
    if (isset($query->sort) && !empty($query->sort)) {
      $queryString .= " ORDER BY ";
      if (is_array($query->sort)) {
        foreach($query->sort as $order => $fieldList) {
          // If more than one field is specified, retrieve them all
          if (is_array($fieldList))
            $queryString .= implode(",", $fieldList)." ".$order;
          // Otherwise, add the field list directly to the query string
          else $queryString .= $fieldList." ".$order;
        }
      }
      else $queryString .= $query->sort;
    }
    // Run the query and return the result
    $query->handle = $this->Query($queryString);
    // Retrieve the number of rows returned by the query
    $rowCount = $this->RowCount($query);
    // If an offset was specified, seek to that offset
    if ($query->offset) {
      $query->handle->Move($query->offset);
      $rowCount -= $query->offset;
    }
    // Indicate the number of rows returned by the query
    return($rowCount);
  }
  
  // Executes a DELETE or TRUNCATE TABLE query on the current database
  function Delete($query, $now = FALSE) {
    // Continue only if we have a valid-looking list of tables
    if (empty($query->location)) return(seInvalidParam);
    // If query criteria was specified, we are deleting one or more rows
    if (isset($query->criteria)) {
      // Build the query string
      $queryString .= "DELETE FROM ";
      // Add the list of tables to the query string
      $queryString .= implode(",", $query->location);
      // Add the query criteria to the query string
      $queryString .= " WHERE ";
      // If the criteria are specified in an array, get them as a string
      if (is_array($query->criteria))
        $queryString .= implode_assoc($query->criteria, " ", " ");
      // Otherwise, add the criteria directly to the query string
      else $queryString .= $query->criteria;
    }
    // Otherwise, if no criteria is specified, the entire table is to be deleted
    else $queryString = "TRUNCATE TABLE ".implode(",", $query->location);
    // Run the query and return the result
    $query->handle = $this->Query($queryString);
    // Indicate the number of rows changed by this query
    return($this->RowsChanged($query));
  }
  
  // Executes a full-text search on the specified tables
  function FindText($query) {
    // Continue only if we have a valid-looking list of tables and criteria
    if (empty($query->location) || empty($query->criteria))
      return(seInvalidParam);
    // Build the query string
    $queryString = "SELECT FT_TABLE.";
    // Add the list of fields to be retrieved from each table
    foreach($query->location as $tableName => $fieldList) {
      // If more than one field is specified, retrieve all of the fields
      if (is_array($fieldList)) $queryString .= implode(", FT_TABLE.", $fieldList);
      // Otherwise, add the field list directly to the string
      else $queryString .= $fieldList;
    }
    $queryString .= ", KEY_TABLE.RANK/10 AS Rank FROM ";
    // Add the list of tables to the query string
    $location = implode(",", array_keys($query->location));
    $queryString .= $location ." AS FT_TABLE";
    // This string holds a name for our table
    $queryString .= " INNER JOIN ";
    // Add the query criteria
    $queryString .= " CONTAINSTABLE(".$location.",*,";
    if (is_array($query->criteria)) $queryString .= Quote(implode("','", $query->criteria));
    else $queryString .= Quote($query->criteria);
    // If a limit was specified, include it
    if (!empty($query->limit)) $queryString .= ", ".$query->limit;
    $queryString .= ") AS KEY_TABLE ON FT_TABLE.ID=KEY_TABLE.[KEY]";
    // If sort criteria was specified, include it
    if (isset($query->sort) && !empty($query->sort)) {
      $queryString .= " ORDER BY ";
      if (is_array($query->sort)) {
        foreach($query->sort as $order => $fieldList) {
          // If more than one field is specified, retrieve them all
          if (is_array($fieldList))
            $queryString .= implode(",", $fieldList)." ".$order;
          // Otherwise, add the field list directly to the query string
          else $queryString .= $fieldList." ".$order;
        }
      }
      else $queryString .= $query->sort;
    }
    if ($query->offset != 0) $queryString .= " OFFSET ".$query->offset;
    // Run the query and return the result
    $query->handle = $this->Query($queryString);
    // Retrieve the number of rows returned by the query
    $rowCount = $this->RowCount($query);
    // Indicate the number of rows returned by the query
    return($rowCount);
  }
  
  // Inserts data into the database
  function Insert($query) {
    /* Continue only if we have a valid-looking list of table names and fields/
       values */
    if (empty($query->location) || empty($query->fieldsValues))
      return(seInvalidParam);
    // There's no need to insert the primary key
    if (isset($query->fieldsValues["ID"])) {
      // Stash the primary key for a moment
      $primaryKey = $query->fieldsValues["ID"];
      // Remove it from the query
      unset($query->fieldsValues["ID"]);
    }
    // Build the query string
    $queryString = "INSERT ";
    $queryString .= implode(",", $query->location);
    /* If the fields/values array is an associative array, we assume that we
       are setting values for specific fields in the table.  Otherwise, the
       fields/values array must contain one value for every field in the table,
       or the query will fail. */
    if (is_assoc($query->fieldsValues)) {
      $queryString .= " ([";
      $queryString .= implode("],[", array_keys($query->fieldsValues));
      $queryString .= "])";
    }
    // Add the values to be inserted
    $queryString .= " VALUES (";
    foreach ($query->fieldsValues as $key => &$value) {
      $value = strtr($value, array(
        "'" => "''")); 
      $value = "'".$value."'";
    }
    $queryString .= implode(",", array_values($query->fieldsValues));
    $queryString .= ")";
    // Run the query and return the result
    $query->handle = $this->Query($queryString);
    // If we had to unset the primary key, we restore it now
    if (isset($primaryKey)) $query->fieldsValues["ID"] = $primaryKey;
    // Indicate the number of rows changed by the query
    return($this->RowsChanged($query));
  }
  
  // Updates data in the specified fields for the specified tables
  function Update($query) {
    // Ensure that we have a valid list of tables, fields/values, and criteria
    if (empty($query->location) || empty($query->fieldsValues) ||
      empty($query->criteria)) return(seInvalidParam);
    // There's no need to update the primary key
    if (isset($query->fieldsValues["ID"])) {
      // Stash the primary key for a moment
      $primaryKey = $query->fieldsValues["ID"];
      // Remove it from the query
      unset($query->fieldsValues["ID"]);
    }
    // Build the query string
    $queryString = "UPDATE ";
    $queryString .= implode(",", $query->location);
    // Include the fields to be updated
    foreach ($query->fieldsValues as $key => &$value) {
      $value = strtr($value, array(
        "'" => "''"));
      $value = "'".$value."'";
    }
    $queryString .= " SET [";
    $queryString .= implode_assoc($query->fieldsValues, ",[", "]=");
    // Include the criteria
    $queryString .= " WHERE ";
    // If the criteria are specified as an array, get them as a string
    if (is_array($query->criteria))
      $queryString .= implode_assoc($query->criteria, " ", " ");
    // Otherwise, add the criteria directly to the query string
    else $queryString .= $query->criteria;
    // Run the query and return the result
    $query->handle = $this->Query($queryString);
    // If we had to unset the primary key, we restore it now
    if (isset($primaryKey)) $query->fieldsValues["ID"] = $primaryKey;
    // Return the number of rows affected by the query
    return($this->RowsChanged($query));
  }
  
  // Returns the last primary key generated by the database server
  function LastKey() {
    // Query for the last key generated
    $result = $this->handle->Execute("SELECT @@IDENTITY");
    // Retrieve the row that contains the last primary key
    return($result->Fields->Item(0)->Value);
 }
  
  // Retrieves the total rows returned by the specified query
  function RowCount($query) {
    // Retrieve the row count
    $rowCount = ($query->handle->State ? $query->handle->RecordCount:0);
    // Output a debug message (if debugging)
    DebugOut("QUERY", $rowCount." row(s) returned");
    // Return the row count
    return($rowCount);
  }
  
  // Retrieve the total number of rows changed by the specified query
  function RowsChanged($query) {
    $result = $this->handle->Execute("SELECT @@ROWCOUNT");
    $rowCount = ($result ? $result->Fields->Item(0)->Value : 0);
    // Output a debug message (if debugging)
    DebugOut("QUERY", $rowCount." row(s) affected");
    // Return the row count
    return($rowCount);
  }
  
  // Retrieves the next row from the specified query
  function NextRow($query) {
    // Ensure the query handle is valid
    if ($query->handle->EOF) return(FALSE);
    // Fetch the next row
    for ($i=0; $i < $query->handle->Fields->Count; $i++) {
      $queryRow[$i] = $query->handle->Fields->Item($i)->Value;
    }
    $query->handle->MoveNext();
    /* Remove white space from each field value (this is a peculiarity of MS
       SQL server when dealing with empty fields) */
    if (is_array($queryRow)) foreach($queryRow as $fieldName => $fieldValue) {
      $queryRow[$fieldName] = trim($fieldValue, " []");
    }
    // Output a debug message (if debugging)
    DebugOut("QUERY", "Fetched row: ".implode(",", $queryRow));
    // Return the row
    return($queryRow);
  }
  
  /* Retrieves the next row from the specified query (both field names and
     values) */
  function NextRowVerbose($query) {
    // Ensure the query handle is valid
    if ($query->handle->EOF) return(FALSE);
    // Fetch the next row
    for ($i=0; $i < $query->handle->Fields->Count; $i++) {
      $queryRow[$query->handle->Fields->Item($i)->Name] = $query->handle->Fields->Item($i)->Value;
    }
    $query->handle->MoveNext();
    /* Remove white space from each field value (this is a peculiarity of MS
       SQL server when dealing with empty fields) */
    if (is_array($queryRow)) foreach($queryRow as $fieldName => $fieldValue) {
      $queryRow[$fieldName] = trim($fieldValue, " []");
    }
    // Output a debug message (if debugging)
    DebugOut("QUERY", "Fetched row: ".implode_assoc($queryRow, "', ","='")."'");
    // Return the row
    return($queryRow);
  }
  
  /* Frees a query from memory */
  function FreeQuery($query) {
    // Ensure the query handle is valid
    return;
  }
  
  /* Retrieves the names of all of the tables in the database */
  function TableNames() {
    // Query on the table names
    $result = $this->Query("SELECT name FROM sysobjects WHERE xtype='U'");
    // If the query failed, return an error
    if (!result) return($result);
    // Create an array to hold the table names
    $tableNames = array();
    // Retrieve each table name
    $queryRow = $result->fetch(PDO::FETCH_NUM);
    while ($queryRow) {
      // Stash the table name
      $tableNames[] = $queryRow[0];
      // Get the next row
      $queryRow = @mssql_fetch_row($result);
    }
    // Return the list of table names
    return($tableNames);
  }
  
  /* Retrieves the names of the fields in the specified table */
  function FieldNames($tableName) {
    // Continue only if we have a valid-looking table name
    if (empty($tableName)) return(seInvalidParam);
    // Run a dummy query on the table
    $dummy = $this->Query("SELECT * FROM ".$tableName." LIMIT 1");
    // If the query failed, indicate an error
    if (!$dummy) return($dummy);
    // Retrieve the field names
    $dummyRow = $dummy->fetch(PDO::FETCH_ASSOC);
    $fieldNames = array_keys($dummyRow);
    // Free the query
    unset($dummy);
    // Return the field names
    return($fieldNames);
  }
  
  /* Converts a PHP time value into a string that the database can handle.  If
     no time value is specified, the current time is used. */
  function TimeValue($time = -1) {
    if ($time < 0) $time = time();
    return(strftime("%m/%d/%Y %H:%M:%S", $time));
  }
}

?>
