<?php

/* mysql.lib.php
   Provides an interface to a MySQL server instance.
   Part of Siteation.
   Copyright (C) 2005 Studio 703.  All rights reserved.
*/

require_once("site.error.php");
require_once("datasource.lib.php");
require_once("misc.lib.php");

// This class interfaces with a Microsoft SQL server instance
class SQLiteDataSource extends DATA_SOURCE {
  // Constructor
  function SQLiteDataSource($host, $database) {
    // Store the name of the database server
    $this->host = $host;
    // Store the name of the database to be accessed
    $this->dataSource = $database;
  }

  // Connects to the SQL server using the specified user name and password
  function Connect($userName, $password) {
    // Try to open the database connection
    try {
      DebugOut("DATASOURCE", "Connecting to ".$this->host.":".$this->dataSource);
      $this->handle = new PDO('sqlite:'.$this->dataSource);
      DebugOut("DATASOURCE", "Successfully connected to ".$this->host);
      DebugOut("DATASOURCE", "Successfully opened database ".$this->dataSource);
    }
    catch (PDOException $e) {
      // If we failed, return an error
      DiePrettily(seDataSource, array($this->host, $this->dataSource, $e->getMessage()));
    }
  }

  // Executes the specified query string
  function Query($queryString) {
    // Ensure we have a valid-looking query string
    if (empty($queryString)) return(seInvalidParam);
    // Stash the query string for future reference
    $this->queryString = $queryString;
    // Start timing the query
    $queryStart = microt(TRUE);
    // Send the query string to the server
    $this->queryResult = $this->handle->query($this->queryString);
    // Stop timing the query
    $this->queryRunTime = round(microt(TRUE) - $queryStart, 4);
    // Output a debug message (if we're debugging)
    DebugOut("QUERY ".$this->queryCount, "[".$this->queryRunTime." second(s)]: ".Quote($queryString,"")." returned ".($this->queryResult ? $this->queryResult:"FALSE\n".implode($this->handle->errorInfo())));
    // If we failed, indicate an error
    if (!$this->queryResult) return(seQuery);
    // Update the query counter
    $this->queryCount++;
    // Return success
    return($this->queryResult);
  }

  // Executes a SELECT query on the current database
  function Select($query) {
    // Continue only if we have a valid looking list of tables to search
    if (empty($query->location)) return(seInvalidParam);
    // Start constructing the query string
    // Add the list of tables to the query string
    $queryString = "FROM ".implode(",", array_keys($query->location));
    // If filter criteria was specified for this search, include it
    if (isset($query->criteria) && !empty($query->criteria)) {
      $queryString .= " WHERE ";
      // If the criteria are specified in an array, get them as a string
      if (is_array($query->criteria))
        $queryString .= implode_assoc($query->criteria, " ", " ");
      // Otherwise, add the criteria directly to the query string
      else $queryString .= $query->criteria;
    }
    // If sort criteria was specified, include it
    if (isset($query->sort) && !empty($query->sort)) {
      $queryString .= " ORDER BY ";
      if (is_array($query->sort)) {
        foreach($query->sort as $order => $fieldList) {
          // If more than one field is specified, retrieve them all
          if (is_array($fieldList))
            $queryString .= implode(",", $fieldList)." ".$order;
          // Otherwise, add the field list directly to the query string
          else $queryString .= $fieldList." ".$order;
        }
      }
      else $queryString .= $query->sort;
    }
    // If an offset or limit was specified, include them
    if (!empty($query->offset)) {
      if (empty($query->limit)) $queryString .= " LIMIT -1 OFFSET ".$query->offset;
      else $queryString .= " LIMIT ".$query->limit." OFFSET ".$query->offset;
    }
    // Strip brackets from the query string (legacy of trying to be compatible
    // with MSSQL server
    $queryString = strtr($queryString, array(
      "[" => "",
      "]" => ""));
    // Find out how many rows will be returned by this query
    $result = $this->Query("SELECT count(*) ".$queryString);
    $rowCount = $result->fetchColumn();
    if ($rowCount) {
      // Add the list of fields to be retrieved from each table
      foreach($query->location as $tableName => $fieldList) {
        // If more than one field is specified, retrieve all of the fields
        if (is_array($fieldList)) $queryString = implode(",", $fieldList)." ".$queryString;
        // Otherwise, add the field list directly to the string
        else $queryString = $fieldList." ".$queryString;
      }
      // Run the query and return the result
      $queryString = "SELECT ".$queryString;
      $query->handle = $this->Query($queryString);
      $query->runTime = $this->queryRunTime;
    }
    // Indicate the number of rows returned by the query
    return($rowCount);
  }

  // Executes a DELETE or TRUNCATE TABLE query on the current database
  function Delete($query, $now = FALSE) {
    // Continue only if we have a valid-looking list of tables
    if (empty($query->location)) return(seInvalidParam);
    // If query criteria was specified, we are deleting one or more rows
    if (isset($query->criteria)) {
      // Build the query string
      $queryString .= "DELETE ";
      // Add the list of tables to the query string
      $queryString .= " FROM ".implode(",", $query->location);
      // Add the query criteria to the query string
      $queryString .= " WHERE ";
      // If the criteria are specified in an array, get them as a string
      if (is_array($query->criteria))
        $queryString .= implode_assoc($query->criteria, " ", " ");
      // Otherwise, add the criteria directly to the query string
      else $queryString .= $query->criteria;
    }
    // Otherwise, if no criteria is specified, the entire table is to be deleted
    else $queryString = "DELETE FROM ".implode(",", $query->location);
    // Run the query and return the result
    $query->handle = $this->Query($queryString);
    // Indicate the number of rows changed by this query
    return($this->RowsChanged($query));
  }

  // Executes a full-text search on the specified tables
  function FindText($query) {
    // Continue only if we have a valid-looking list of tables and criteria
    if (empty($query->location) || empty($query->criteria))
      return(seInvalidParam);
    // Build the query string
    $queryString = "SELECT ";
    // Add the list of fields to be retrieved from each table
    foreach($query->location as $tableName => $fieldList) {
      // If more than one field is specified, retrieve all of the fields
      if (is_array($fieldList)) $queryString .= implode(",", $fieldList);
      // Otherwise, add the field list directly to the string
      else $queryString .= $fieldList;
    }
    // Build the match string
    $match = "'";
    if (is_array($query->criteria)) $match .= Quote(implode("','", $query->criteria));
    else $match .= $query->criteria;
    $match .= "' IN (";
    if (is_array($query->fieldsValues)) $match .= implode(",", $query->fieldsValues);
    else $match .= $query->fieldsValues;
    $match .= ")";
    // Add the list of tables to the query string
    $queryString .= " FROM ".implode(",", array_keys($query->location));
    // Add the query criteria
    $queryString .= " WHERE".$match;
    // If sort criteria was specified, include it
    if (isset($query->sort) && !empty($query->sort)) {
      $queryString .= " ORDER BY ";
      if (is_array($query->sort))
        foreach($query->sort as $order => $fieldList) {
          // If more than one field is specified, retrieve them all
          if (is_array($fieldList))
            $queryString .= implode(",", $fieldList)." ".$order;
          // Otherwise, add the field list directly to the query string
          else $queryString .= $fieldList." ".$order;
        }
      else $queryString .= $query->sort;
    }
    // If a limit or offset was specified, include it
    if ($query->limit > 0) $queryString .= " LIMIT ".$query->limit;
    if ($query->offset != 0) $queryString .= " OFFSET ".$query->offset;
    // Run the query and return the result
    $query->handle = $this->Query($queryString);
    $query->runTime = $this->queryRunTime;
    // Indicate the number of rows returned by the query
    return($this->RowCount($query));
  }

  // Inserts data into the database
  function Insert($query) {
    /* Continue only if we have a valid-looking list of table names and fields/
       values */
    if (empty($query->location) || empty($query->fieldsValues))
      return(seInvalidParam);
    // There's no need to insert the primary key
    if (isset($query->fieldsValues["ID"])) {
      // Stash the primary key for a moment
      $primaryKey = $query->fieldsValues["ID"];
      // Remove it from the query
      unset($query->fieldsValues["ID"]);
    }
    // Build the query string
    $queryString = "INSERT INTO ";
    $queryString .= implode(",", $query->location);
    /* If the fields/values array is an associative array, we assume that we
       are setting values for specific fields in the table.  Otherwise, the
       fields/values array must contain one value for every field in the table,
       or the query will fail. */
    if (is_assoc($query->fieldsValues)) {
      $queryString .= " (";
      $queryString .= implode(",", array_keys($query->fieldsValues));
      $queryString .= ")";
    }
    // Add the values to be inserted
    $queryString .= " VALUES ('";
    $queryString .= implode("','", array_values($query->fieldsValues));
    $queryString .= "')";
    // Run the query and return the result
    $query->handle = $this->Query($queryString);
    // If we had to unset the primary key, we restore it now
    if (isset($primaryKey)) $query->fieldsValues["ID"] = $primaryKey;
    // Indicate the number of rows changed by the query
    return($this->RowsChanged($query));
  }

  // Updates data in the specified fields for the specified tables
  function Update($query) {
    // Ensure that we have a valid list of tables, fields/values, and criteria
    if (empty($query->location) || empty($query->fieldsValues) ||
      empty($query->criteria)) return(seInvalidParam);
    // There's no need to update the primary key
    if (is_array($query->fieldsValues) && isset($query->fieldsValues["ID"])) {
      // Stash the primary key for a moment
      $primaryKey = $query->fieldsValues["ID"];
      // Remove it from the query
      unset($query->fieldsValues["ID"]);
    }
    // Build the query string
    $queryString = "UPDATE ";
    $queryString .= implode(",", $query->location);
    // Include the fields to be updated
    $queryString .= " SET ";
    if (is_array($query->fieldsValues))
      $queryString .= implode_assoc($query->fieldsValues, "\",", "=\"")."\"";
    else $queryString .= $query->fieldsValues;
    // Include the criteria
    $queryString .= " WHERE ";
    // If the criteria are specified as an array, get them as a string
    if (is_array($query->criteria))
      $queryString .= implode_assoc($query->criteria, " ", " ");
    // Otherwise, add the criteria directly to the query string
    else $queryString .= $query->criteria;
    // Run the query and return the result
    $query->handle = $this->Query($queryString);
    // If we had to unset the primary key, we restore it now
    if (isset($primaryKey)) $query->fieldsValues["ID"] = $primaryKey;
    // Return the number of rows affected by the query
    return($this->RowsChanged($query));
  }

  // Returns the last primary key generated by the database server
  function LastKey() {
    // Query for the last key generated
    $result = $this->Query("SELECT last_insert_rowid()");
    // If we failed, indicate an error
    if ($result < 0) return($result);
    // Retrieve the row that contains the last primary key by the server
    return($result->fetchColumn());;
  }

  // Retrieves the total rows returned by the specified query
  function RowCount($query) {
    return(FALSE);
  }

  // Retrieve the total number of rows changed by the specified query
  function RowsChanged($query) {
    if (!is_object($query->handle)) return(seInvalidParam);
    // Retrieve the number of rows affected
    $rowCount = $query->handle->rowCount();
    // Output a debug message (if debugging)
    DebugOut("QUERY", $rowCount." row(s) affected");
    // Return the row count
    return($rowCount);
  }

  // Retrieves the next row from the specified query
  function NextRow($query) {
    // Ensure the query handle is valid
    if (!is_object($query->handle)) return(FALSE);
    // Fetch the next row
    $queryRow = $query->handle->fetch(PDO::FETCH_NUM);
    // Output a debug message (if debugging)
    DebugOut("QUERY", "Fetched row: ".@implode(",", $queryRow));
    // Return the row
    return($queryRow);
  }

  /* Retrieves the next row from the specified query (both field names and
     values) */
  function NextRowVerbose($query) {
    // Ensure the query handle is valid
    if (!is_object($query->handle)) return(FALSE);
    // Fetch the next row
    $queryRow = $query->handle->fetch(PDO::FETCH_ASSOC);
    // Output a debug message (if debugging)
    DebugOut("QUERY", "Fetched row: ".implode_assoc($queryRow, "', ","='")."'");
    // Return the row
    return($queryRow);
  }

  /* Frees a query from memory */
  function FreeQuery($query) {
    // Ensure the query handle is valid
    if (!$query->handle) return(seInvalidParam);
    $query->handle = null;
  }

  /* Retrieves the names of all of the tables in the database */
  function TableNames() {
    // Query on the table names
    $result = $this->Query("SELECT name FROM (SELECT * FROM sqlite_master UNION ALL SELECT * FROM sqlite_temp_master) WHERE type='table' ORDER BY name");
    // If the query succeeds, $result will be an array of table names
    return($result->fetchAll());
  }

  /* Retrieves the names of the fields in the specified table */
  function FieldNames($tableName) {
    // Continue only if we have a valid-looking table name
    if (empty($tableName)) return(seInvalidParam);
    // Run a dummy query on the table
    $dummy = $this->Query("SELECT * FROM ".$tableName." LIMIT 1");
    // If the query failed, indicate an error
    if ($dummy < 0) return($dummy);
    // Retrieve the field names
    $dummyRow = @mysql_fetch_assoc($dummy);
    $fieldNames = array_keys($dummyRow);
    // Free the query
    @mysql_free_result($dummy);
    // Return the field names
    return($fieldNames);
  }

  /* Converts a PHP time value into a string that the database can handle.  If
     no time value is specified, the current time is used. */
  function TimeValue($time = -1) {
    if ($time < 0) $time = time();
    return(strftime("%Y-%m-%d %H:%M:%S", $time));
  }
}

?>
