<?php

/* user.lib.php
   Defines a class used to manage Siteation users.
   Part of Siteation.
   Copyright (C) 2005 Studio 703.  All rights reserved.
*/

require_once("site.error.php");
require_once("query.lib.php");
require_once("site.lib.php");

// This class handles Siteation users
class SITE_USER {
  // Stores the site handler used to manage the site
  private $site;
  // Stores the data source used to manage the user information
  private $dataSource;
  /* Indicates whether or not the user information has changed since it was
     last read from or written to the database */
  private $changed;
  // Stores fields retrieved from the user's record
  private $settings;
  // Stores groups to which the user belongs
  private $groups;

  // Constructor
  function __construct($siteHandler) {
    // Continue only if we have a valid-looking site handler
    if (!isset($siteHandler)) return(seInvalidParam);
    // Store the site handler for future use
    $this->site = $siteHandler;
    // Store the site data source handler for future use
    $this->dataSource = $this->site->DataSource();
  }

  // Frees the user handler and flushes any changes to the database
  function __destruct() {
    // Flush any changes to the database
    $this->FlushChanges();
  }

  /* Retrieves the specified user setting. */
  function Setting($settingName) {
    // Continue only if we have a valid setting
    if (!isset($this->settings[$settingName]))
      return(NULL);
    // Return the requested setting
    return($this->settings[$settingName]);
  }

  // Sets the specified user setting
  function Set($settingName, $value) {
    // Continue only if we have a valid-looking setting
    if (empty($settingName))   return(seInvalidParam);
    // Store the new setting
    $this->settings[$settingName] = $value;
    // Mark the user information as changed
    $this->changed = TRUE;
    // Return success
    return(seOkay);
  }

  // Retrieves all of the user's settings at once
  function Settings() {
    // Return all settings
    return($this->settings);
  }

  // Sets all of the user's settings at once
  function SetAll($settings) {
    // Store the new settings
    $this->settings = $settings;
    // Mark the information as changed
    $this->changed = TRUE;
  }

  // Creates a record for the specified user
  function Create($userName) {
    // If no username was specified, exit
    if (empty($userName)) return(seInvalidParam);
    global $SITE_CONFIG;
    // Create a query
    $user = new QUERY($this->dataSource);
    // Select the table to update
    $user->location = $SITE_CONFIG["TABLE_PREFIX"]."users";
    // Stash the user name
    $this->settings["UserName"] = $userName;
    $user->fieldsValues = $this->settings;
    // Go!
    $result = $user->Insert();
    // If we failed to create the user, indicate an error
    if ($result < 0) return(seQuery);
    // Attach ourselves to the new user record
    $this->settings["ID"] = $this->dataSource->LastKey();
  }
  
  // Retrieves the specified user from the database
  function Get($userName) {
    // Continue only if we have a valid-looking user name
    if (empty($userName)) return(seInvalidParam);
    global $SITE_CONFIG;
    // Create a query
    $user = new QUERY($this->dataSource);
    // Select the table to search
    $user->location = array(
      $SITE_CONFIG["TABLE_PREFIX"]."users" => "*");
    // Set the criteria
    $user->criteria = array(
      "UserName=", Quote($userName));
    // Run the query
    $result = $user->Select();
    // If the query failed, indicate that failure
    if ($result <= 0) return($result);
    // Retrieve the row containing the user's information
    $this->settings = $user->NextRowVerbose();
    // Mark the user's information as unchanged
    $this->changed = FALSE;
    // Free the query
    $user->__destruct();
    // Expand the user's icons
    $this->settings["Icons"] = explode("|", $this->settings["Icons"]);
    // Indicate success
    return($this->settings["ID"]);
  }

  // Flushes session information to the database
  function FlushChanges() {
    // Continue only if changes were made
    if (!$this->changed) return(seOkay);
    global $SITE_CONFIG;
    // Create a query to update the user's information
    $user = new Query($this->dataSource);
    // Set the table on which to operate
    $user->location = array($SITE_CONFIG["TABLE_PREFIX"]."users");
    // Set the fields to be updated
    foreach($this->settings as $keyName => $keyValue) {
      // Collapse any arrays in the settings array
      if (is_array($keyValue)) 
        $user->fieldsValues[$keyName] = implode("|", $keyValue);
      else $user->fieldsValues[$keyName] = $keyValue;
    }
    // Set the criteria so that the correct user record is updated
    $user->criteria = array(
      "ID=", $this->settings["ID"]);
    // Run the query
    $result = $user->Update();
    // If the query failed, return the error
    if ($result < 0) return($result);
    // Mark the user's data as unchanged
    $this->changed = FALSE;
    // Return success
    return(seOkay);
  }

  // Retrieves the list of groups to which the user belongs
  function Membership($reload = false) {
    global $SITE_CONFIG;
    if ((empty($this->groups)) || ($reload)) {
      // Query the 'user2group' table to discover the group membership for the user
      $groups = new Query($this->dataSource);
      // Set the table on which to operate
      $groups->location = array($SITE_CONFIG["TABLE_PREFIX"]."user2group" => "*");
      // Set the search criteria
      $groups->criteria = "UserName=".Quote($this->settings["UserName"]);
      // Run the search
      $result = $groups->Select();
      // If no membership information exists, return nothing
      if ($result <= 0) return(NULL);
      // Get each group
      $resultRow = $groups->NextRowVerbose();
      while($resultRow) {
        $this->groups[$resultRow["GroupName"]] = true;
        $resultRow = $groups->NextRowVerbose();
      }
      // Free the query
      $groups->__destruct();
    }
    // Return the group list
    return($this->groups);
  }
  
  // Sets the groups to which the user belongs
  function SetMembership($groupList) {
    // Continue only if a list of groups was given
    if (empty($groupList)) return(seInvalidParam);
    // Set the new group membership information
    if (is_array($groupList)) $this->Set("Groups", implode("|", $groupList));
    else $this->Set("Groups", $groupList);
    // Return success
    return(seOkay);
  }

  // Determines whether or not the user is a member of the specified group
  function MemberOf($groupName) {
    // If no group name was specified, return FALSE
    if (empty($groupName)) return(FALSE);
    // Search the group list for the group name
    $return(array_key_exists($groupName, $this->groups));
  }
  /* Determines whether or not the user is subscribed to a particular channel,
     based on which groups they belong to.  If the user belongs to a group that
     is subscribed to the channel, that subscription is returned. */
  function Subscription($channelHandler) {
    global $SITE_CONFIG;
    // Continue only if we have a valid-looking channel handler
    if (!isset($channelHandler)) return(NULL);
    // Retrieve group membership for this user
    $groups = $this->Membership();
    /* If nothing was returned, then the user belongs to no groups and therefore
       is subscribed to nothing */
    if ($groups == NULL) return(NULL);
    // Convert the list of groups to a comma-delimited string for use in a query
    $groups = Quote(implode("','", array_keys($groups)));
    // Build a list of channels to search
    if (!is_string($channelHandler)) {
      $channels = Quote($channelHandler->Setting("ChannelName"));
    /* Since group subscriptions may specify that they apply to all @PUBLIC or
       all @PRIVATE channels, we need to allow for this, based on whether or
       not the channel on which we are basing our search is public or private */
      $channels .= ", ";
      $channels .=
        ($channelHandler->Setting("Public") == "Y") ? "'@PUBLIC'":"'@PRIVATE'";
    }
    else $channels = Quote($channelHandler);
    // Create a query to search for the subscription
    $subscription = new QUERY($this->dataSource);
    // Select the table to search
    $subscription->location = array(
      $SITE_CONFIG["TABLE_PREFIX"]."subscriptions" => "*");
    // Set the search criteria
    $subscription->criteria = array(
      "GroupName IN ", Quote($groups, "()"), " AND ",
      "ChannelName IN ", Quote($channels, "()"));
    // Set the sort order
    $subscription->sort = array(
      "DESC" => "ChannelName");
    // Run the query
    $result = $subscription->Select();
    // If the query failed, return the error
    if ($result < 0) return($result);
    // If the query returned no rows, the user is not subscribed to this channel
    if ($result == 0) return(seResourceForbidden);
    // Otherwise, retrieve the subscription
    $resultRow = $subscription->NextRowVerbose();
    // Free the query
    $subscription->__destruct();
    // Return the subscription
    return($resultRow);
  }
  
  // Retrieves the user-specific view record for a channel, if one is specified
  function GetView($channelName) {
    // Continue only if we are working with a valid-looking channel name
    if (empty($channelName)) return(NULL);
    global $SITE_CONFIG;
    // Construct a query to search for the view
    $view = new QUERY($this->dataSource);
    // Set the table to be searched
    $view->location = array(
      $SITE_CONFIG["TABLE_VIEWS"] => "*");
    // Set the criteria for the query
    $view->criteria = array(
      "ChannelName" => "=".Quote($channelName)." AND",
      "UserName" => "=".Quote($this->settings["UserName"]));
    // Run the query
    $result = $view->Select();
    // If the query failed, indicate that no views were found
    if ($result <= 0) return(NULL);
    // Otherwise, return the first row found
    $resultRow = $view->NextRowVerbose();
    // Free the query
    $view->__destruct();
    // Return the result row
    return($resultRow);
  }
}

?>
