<?php

/* view.lib.php
   Defines a class used to handle Siteation views.
   Part of Siteation.
   Copyright (C) 2005 Studio 703.  All rights reserved.
*/

require_once("smart.lib.php");
require_once("query.lib.php");
require_once("post.lib.php");
require_once("site.error.php");
require_once("view.modifiers.php");

// This class handles Views
class SITE_VIEW extends SMART_SITE {
  // Stores the site handler used to manage the site
  private $site;
  // Stores the data source used to manage the view
  private $view;
  // Stores the session handler used to manage the session
  private $session;
  // Stores the user handler used to manage the user
  private $user;
  // Stores the channel handler used to manage the channel
  private $channel;
  // Stores the data retrieved from the View's record
  private $settings;
  /* Indicates whether or not any view settings have changed since they were
     last read from/written to the database. */
  private $changed;
  // Stores extra parameters passed to the view (used for dynamic views)
  public $params;

  // Constructor
  function __construct($site, $session, $channel, $user) {
    // Continue only if we have been given valid handlers
    if (!isset($site)) DiePrettily(seInvalidParam, $this);
    if (!isset($session)) DiePrettily(seInvalidParam, $this);
    if (!isset($channel)) DiePrettily(seInvalidParam, $this);
    if (!isset($user)) DiePrettily(seInvalidParam, $this);
    // Call the parent constructor
    parent::__construct();
    // Stash the handlers for future use
    $this->site = $site;
    $this->dataSource = $this->site->DataSource();
    $this->session = $session;
    $this->channel = $channel;
    $this->user = $user;
    // Set up caching if the site options specify it
    $this->caching =
      ($this->site->Setting("Caching", "CacheViews") == "N") ? 0:2;
    $this->cacheID = "";
    $this->cacheTTL = $this->site->Setting("Caching", "CacheTTL");
    $this->cache_handler_func = "Cache_Handler";
    $this->changed = array();
  }
  
    // Frees the view handler and flushes any changes to the database
  function __destruct() {
    // Update the caching information
    if ($this->IsCached())
      $this->Set("LastCached", $this->dataSource->TimeValue());
    // Update the view count
    $viewCount = $this->settings["ViewCount"] + 1;
    $this->Set("ViewCount", $viewCount);
    // Update the channel view count
    $viewCount = $this->channel->Setting("FetchCount") + 1;
    $this->channel->Set("FetchCount", $viewCount);
    // Flush any changes to the database
    $this->FlushChanges();
  }

  /* Retrieves the channel setting */
  function Channel() {
    // Return the channel handler for this view
    return ($this->channel);
  }
  
  /* Retrieves the specified view setting. */
  function Setting($settingName) {
    // Continue only if we have a valid setting
    //if (is_array($settingName)) $settingName = $settingName[0];
    if (empty($settingName) || !isset($this->settings[$settingName]))
      return(NULL);
    // Return the requested setting
    return($this->settings[$settingName]);
  }

  // Sets the specified view setting
  function Set($settingName, $value) {
    // Continue only if we have a valid-looking setting
    if (empty($settingName)) return(seInvalidParam);
    // Store the new setting
    $this->settings[$settingName] = $value;
    // Mark the channel information as changed
    $this->changed[] = $settingName;
    // Return success
    return(seOkay);
  }

  // Retrieves all of the view settings at once
  function Settings() {
    // Return all view settings
    return($this->settings);
  }

  // Sets all of the view settings at once
  function SetAll($settings) {
    // Store the new settings
    $this->settings = $settings;
    // Mark the channel information as changed
    $this->changed = TRUE;
  }
  
  // Gets the view data source
  function DataSource() {
    return $this->dataSource;
  }

  // Retrieves the specified view from the database
  function Get($viewName) {
    /* If no view was specified, return an error */
    if (empty($viewName)) return(seInvalidParam);
    global $SITE_CONFIG;
    // Create a query
    $view = new QUERY($this->dataSource);
    // Select the table to search
    $view->location = array(
      $SITE_CONFIG["TABLE_PREFIX"]."views" => "*");
    // Set the criteria
    /* Specify what channels to search for.  Since views can be applied to
       @ALL channels, or all @PUBLIC or @PRIVATE channels, we need to include
       these views in our search */
    $channelNames = Quote($this->channel->Setting("ChannelName"));
    $channelNames .= ",'<ALL>',";
    $channelNames .=
      ($this->channel->Setting("Public") == "Y") ? "'<PUBLIC>'" : "'<PRIVATE>'";
    $view->criteria = array(
      "ViewName=", Quote($viewName), " AND ",
      "ChannelName IN (". $channelNames.")");
    // Run the query
    $result = $view->Select();
    // If the query failed, indicate that failure
    if ($result <= 0) return(seResourceNotFound);
    // Retrieve the row containing the view information
    $this->settings = $view->NextRowVerbose();
    //$this->settings["Template"] = $SITE_CONFIG["SCRIPT_URL"].$this->settings["Template"];
    // Mark the channel information as unchanged
    $this->changed = FALSE;
    // Free the query
    $view->__destruct();
    // Set up additional variables for caching
    if ($this->caching) {
      // The cache ID is the view id number and name
      $this->cacheID = $this->settings["ID"].$this->settings["ViewName"];
    }
    // Indicate success
    return($this->settings["ID"]);
  }

  // Flushes session information to the database
  function FlushChanges() {
    // Continue only if changes were made
    if (empty($this->changed)) return(seOkay);
    global $SITE_CONFIG;
    // Create a query to update the view information
    $view = new Query($this->dataSource);
    // Set the table on which to operate
    $view->location = array($SITE_CONFIG["TABLE_PREFIX"]."views");
    // Stash the view data
    foreach($this->changed as $change)
      $view->fieldsValues[$change] = $this->settings[$change];
    // Set the criteria so that the correct view record is updated
    $view->criteria = array(
      "ID =", $this->settings["ID"]);
    // Run the query
    $result = $view->Update();
    // If the query failed, return the error
    if ($result <= 0) return($result);
    // Mark the channel data as unchanged
    $this->changed = array();
    // Return success
    return(seOkay);
  }
  
  /* Replaces special "@" tags in a string */
  function ReplaceAtTags($source) {
    return(strtr($source, array(
      "@channelName" => $this->channel->Setting("ChannelName"),
      "@channelPubDate" => $this->channel->Setting("PubDate"),
      "@userName" => $this->user->Setting("UserName"),
      "@viewName" => $this->settings["ViewName"])));
  }
  
  /* Registers the variables that the view template is allowed to use */
  function RegisterVariables() {
    GLOBAL $SITE_CONFIG;
    $this->assign("thisSite", $this->site->Settings());
    $this->assign("thisSession", $_SESSION);
    $this->assign("thisChannel", $this->channel->Settings());
    $userSettings = $this->user->Settings();
    $userSettings["LoggedIn"] = $this->session->LoggedIn();
    $this->assign("thisUser", $userSettings);
    $this->assign("thisView", $this->Settings());
    $templateURL = explode("/", $this->settings["Template"]);
    array_pop($templateURL);
    $templateURL = implode("/", $templateURL);
    $templateURL = $SITE_CONFIG["SCRIPT_URL"].$templateURL;
    $this->assign("global", array(
      "scriptLocation" => $SITE_CONFIG["SCRIPT_URL"],
      "templateLocation" => $templateURL
    ));
    // Register the function to fetch posts
    $this->register_function("fetching", "viewFetchData");
    // Register post "modifiers"
    $this->register_modifier("next", "viewNextRowFrom");
    $this->register_modifier("count", "viewCountResultsIn");
    $this->register_modifier("runtime", "viewRuntimeFor");
    $this->register_modifier("done", "viewDone");     
    // Get dynamic view parameters
    if ((isset($_GET['params'])) && (!empty($_GET['params']))) 
      $this->params = explode_assoc($_GET['params'], ";", ":");
    $this->assign("params", $this->params);
  }
    
  /* Shows the current view */
  function Show() {
    // Continue only if we have retrieved view information
    if (empty($this->settings)) 
      DiePrettily(seDataUnavailable, array(
        get_class($this),
        " - The view handler was unable to retrieve view settings, most likely",
        "due to a configuration error."));
    // Register the objects and functions which the template is allowed to use
    // Register the variables that this template is allowed to use
    $this->RegisterVariables();
    //$this->cache_lifetime = $cacheTTL;
    // Display the view
    $output = $this->fetch("../".$this->settings["Template"], $this->cacheID);
    //$stripSearch = array (
      // Strip {nocache:...} from output
    //  '%{(/)?(nocache:)(.+)}%iU',
      // Strip warning messages from output
    //  '%(<b>Warning)(.+)(<br />)%isU');
    //$output = preg_replace($stripSearch,'<!-- $1$2$3 -->', $output);
    echo $output;
    return(seOkay);
  }

  // Determines whether or not the view is cached
  function IsCached() {
    return (($this->caching > 0) && ($this->is_cached($this->settings["Template"], $this->cacheID)));
  }
}

// Handles caching of the templates
function Cache_Handler($action, $smartView, $content, $template=null, 
  $cacheID=null, $compileID=null, $expTime=null) {
  // The cache file name is the cache ID and the name of the template
  $cacheFileName = $smartView->cache_dir."/".$cacheID;
  $cacheFileName .= "_".basename($template);
  // If we can compress the data, do so
  $compress = false;
  if ((function_exists("gzencode")) && (function_exists("gzdecode"))) {
    $compress = true;
    $cacheFileName .= ".gz";
  }
  $result = false;
  switch($action) {
    // We want to read from a cached file
    case "read":
      $content = "";
      // See if the file exists
      if (!file_exists($cacheFileName)) return(false);
      // Determine if the view has expired
      if (time() > strtotime($smartView->Setting("ViewExpires"))) {
        // Delete the file
        unlink($cacheFileName);
        // Force the cache to be regenerated
        return(false);
      }
      // See if the file is older than the channel's publication date
      $channel = $smartView->Channel();
      if (!$channel) return(false);
      $channelPubDate = strtotime($channel->Setting("PubDate"));
      $viewCacheTTL = $smartView->Setting("CacheTTL");
      if ($channelPubDate >= filemtime($cacheFileName)) {
        // Delete the cached copy of the file
        unlink($cacheFileName);
        $result = false;
      }
      // Otherwise, read from the file
      else {
        $result = file_get_contents($cacheFileName);
        if (!$result) return(false);
        if ($compress) $content = gzdecode($result);
        else $content = $result;
        $result = true;
      }
      break;
    // We want to write to a cache file
    case "write":
      if ($compress) $content = gzencode($content, 9);
      $result = file_put_contents($cacheFileName, $content);
      break;
    // We want to clear a cached file
    case "clear":
      // If the file exists, clear it
      if (file_exists($cacheFileName)) return(unlink($cacheFileName));
      break;
  }
  return($result);
}

?>
